use MediaBase

-- Kapitel 7 --

-- Fallunterscheidung mit CASE

SELECT *, 'Hardcover'
FROM Buch
WHERE Hardcover=1
UNION
SELECT *, 'Softcover'
FROM Buch
WHERE Hardcover=0

SELECT *, CASE
	WHEN Hardcover=1 
	THEN 'Hardcover' 
	ELSE 'Softcover' 
	END AS Cover
FROM Buch

SELECT *, CASE
	WHEN Hardcover=1 THEN 'Hardcover'
	WHEN Hardcover=0 THEN 'Softcover'
	ELSE '(unbekannt)'
	END AS Cover
FROM dbo.Buch

SELECT *, CASE Hardcover
	WHEN 1 THEN 'Hardcover'
	WHEN 0 THEN 'Softcover'
	ELSE '(unbekannt)'
	END AS Cover
FROM dbo.Buch

-- Fallunterscheidung mit IIF und CHOOSE

SELECT *, IIF(Hardcover=1, 'Hardcover', 'Softcover') AS Cover
FROM dbo.Buch

SELECT CHOOSE(2, 'eins', 'zwei', 'drei')

SELECT *, CHOOSE(Laendercode, 'USA', 'Europa/Sdafrika/Naher Osten/Japan', 'Sdost-Asien', 
	'Australien/Mittelamerika/Sdamerika','Osteuropa/ehem.UdSSR/Indien/Afrika','China') AS LaendercodeText
FROM dbo.DVD

SELECT *, CHOOSE(Laendercode+1, 'alle Regionen', 'USA', 'Europa/Sdafrika/Naher Osten/Japan', 'Sdost-Asien', 
	'Australien/Mittelamerika/Sdamerika','Osteuropa/ehem.UdSSR/Indien/Afrika','China') AS LaendercodeText
FROM dbo.DVD

-- Unterabfragen

SELECT idCD
FROM dbo.CD_Track
GROUP BY idCD
HAVING COUNT(*)>13

SELECT Titel, Interpret
FROM dbo.CD
WHERE ID IN
	(SELECT idCD
	FROM dbo.CD_Track
	GROUP BY idCD
	HAVING COUNT(*)>13)

SELECT Titel, Interpret
FROM dbo.CD
WHERE EXISTS
	(SELECT idCD
	FROM dbo.CD_Track
	WHERE idCD = dbo.CD.ID
	GROUP BY idCD
	HAVING COUNT(*)>13)

SELECT Titel, Interpret
FROM dbo.CD
WHERE EXISTS
	(SELECT idCD
	FROM dbo.CD_Track
	GROUP BY idCD
	HAVING COUNT(*)>13 AND idCD = dbo.CD.ID)

SELECT Titel, Interpret
FROM dbo.CD
WHERE (SELECT count(*)
	FROM dbo.CD_Track
	WHERE idCD = dbo.CD.ID)>13

--

SELECT Titel, Interpret, 
	(SELECT count(*)
	FROM dbo.CD_Track
	WHERE idCD = dbo.CD.ID) AS Anzahl
FROM dbo.CD

SELECT CD.Titel, CD.Interpret, count(CD_Track.ID) as Anzahl
FROM dbo.CD 
	LEFT JOIN dbo.CD_Track ON CD.ID=CD_Track.idCD
GROUP BY CD.Titel, CD.Interpret

SELECT CD.Titel, CD.Interpret, TrackAnzahl.Anzahl
FROM dbo.CD
	LEFT JOIN
		(SELECT idCD, count(*) AS Anzahl
		FROM dbo.CD_Track
		GROUP BY idCD) AS TrackAnzahl
	ON CD.ID = TrackAnzahl.idCD

-- GROUP BY ... ROLLUP

SELECT Interpret, count(*) as Anzahl
FROM dbo.CD 
GROUP BY Interpret

SELECT 'Summe', count(*) as Anzahl
FROM dbo.CD 

SELECT Interpret, count(*) as Anzahl
FROM dbo.CD 
GROUP BY Interpret
UNION
SELECT 'Summe', count(*) as Anzahl
FROM dbo.CD 

-- alte Syntax
SELECT Interpret, count(*) as Anzahl
FROM dbo.CD 
GROUP BY Interpret WITH ROLLUP 

-- neue ISO-konforme Syntax
SELECT Interpret, count(*) as Anzahl
FROM dbo.CD 
GROUP BY ROLLUP (Interpret)

SELECT Interpret, Titel, count(*) as Anzahl
FROM dbo.CD 
GROUP BY ROLLUP (Interpret, Titel)

SELECT Interpret, count(*) as Anzahl, min(Erscheinungsjahr) as Von, max(Erscheinungsjahr) as Bis
FROM dbo.CD 
GROUP BY ROLLUP (Interpret)

-- Komplexe INSERTs, UPDATEs & DELETEs

TRUNCATE TABLE CDMitTracks

INSERT INTO CDMitTracks
SELECT CD.*, CDT.TrackNr, CDT.Titel as TrackTitel
FROM CD LEFT JOIN CD_Track as CDT ON CD.ID = CDT.idCD

--

UPDATE VW_AktuelleCDs
SET TrackBewertung=Bewertung
WHERE TrackBewertung IS NULL

UPDATE dbo.CD_Track
SET CD_Track.Bewertung=CD.Bewertung
FROM dbo.CD 
	INNER JOIN dbo.CD_Track ON CD.ID=CD_Track.idCD
WHERE CD_Track.Bewertung IS NULL

UPDATE dbo.CD_Track
SET Bewertung=(SELECT Bewertung FROM dbo.CD WHERE CD.ID=CD_Track.idCD)
WHERE CD_Track.Bewertung IS NULL

--

SELECT * INTO dbo.CD2 FROM dbo.CD

DELETE FROM dbo.CD2
FROM dbo.CD2 LEFT JOIN dbo.CD_Track ON CD2.ID = CD_Track.idCD
WHERE CD_Track.idCD IS NULL

DROP TABLE dbo.CD2
SELECT * INTO dbo.CD2 FROM dbo.CD

DELETE dbo.CD2
FROM dbo.CD2 LEFT JOIN dbo.CD_Track ON CD2.ID = CD_Track.idCD
WHERE CD_Track.idCD IS NULL

DROP TABLE dbo.CD2
SELECT * INTO dbo.CD2 FROM dbo.CD

DELETE FROM dbo.CD2
WHERE ID NOT IN (SELECT DISTINCT idCD FROM dbo.CD_Track)

DROP TABLE dbo.CD2

-- MERGE

SELECT Top 1 *
INTO dbo.BuchImport
FROM dbo.Buch

UPDATE dbo.BuchImport
SET Kategorie = 'Fachbuch'

INSERT INTO dbo.BuchImport (Autor, Titel, Kategorie)
VALUES ('Robert Panther', 'Das Pocket PC Programmierhandbuch', 'Fachbuch')

SELECT * FROM dbo.BuchImport

-- Alte Variante mit update, insert & delete:
UPDATE dbo.Buch
SET Buch.Autor = BuchImport.Autor,
	Buch.Titel = BuchImport.Titel,
	Buch.Kategorie = BuchImport.Kategorie
FROM dbo.Buch
	INNER JOIN dbo.BuchImport ON Buch.Titel=BuchImport.Titel

INSERT INTO dbo.Buch (Autor, Titel, Kategorie)
SELECT Autor, Titel, Kategorie
FROM dbo.BuchImport
WHERE Titel NOT IN (SELECT Titel FROM dbo.Buch)

DELETE FROM dbo.Buch
WHERE Titel NOT IN (SELECT Titel FROM dbo.BuchImport)

-- Neue Variante mit merge:
SELECT * FROM dbo.Buch
SELECT * FROM dbo.BuchImport

MERGE INTO dbo.Buch
USING BuchImport ON Buch.Titel=BuchImport.Titel
WHEN MATCHED
	THEN UPDATE SET Buch.Autor = BuchImport.Autor,
		Buch.Titel = BuchImport.Titel,
		Buch.Kategorie = BuchImport.Kategorie
WHEN NOT MATCHED
	THEN INSERT (Autor, Titel, Kategorie) 
	VALUES (BuchImport.Autor, BuchImport.Titel, BuchImport.Kategorie)
WHEN NOT MATCHED BY SOURCE THEN
	DELETE;

SELECT * FROM dbo.Buch

-- Common Table Expressions --

WITH CTE AS
(
	SELECT idCD, count(*) AS AnzahlSongs
	FROM dbo.CD_Track
	GROUP BY idCD
)
SELECT CD.Interpret, CD.Titel, CTE.AnzahlSongs
FROM dbo.CD as CD
	INNER JOIN CTE on CD.id=CTE.idCD

SELECT CD.Interpret, CD.Titel, count(*) AS AnzahlSongs
FROM dbo.CD as CD
	INNER JOIN dbo.CD_Track AS CDT on CD.id=CDT.idCD
GROUP BY CD.Interpret, CD.Titel

-- Offset-Klausel --
GO

WITH CTE1 AS
(
	SELECT TOP 20 *
	FROM dbo.CD_Track
	ORDER BY ID 
),
CTE2 AS
(
	SELECT TOP 10 *
	FROM CTE1 
	ORDER BY ID DESC
)
SELECT *
FROM CTE2
ORDER BY ID

SELECT *
FROM dbo.CD_Track
WHERE ID BETWEEN 11 AND 20

SELECT *
FROM dbo.CD_Track
ORDER BY ID 
	OFFSET 10 ROWS
	FETCH NEXT 10 ROWS ONLY

-- Sequenzen --

CREATE SEQUENCE dbo.MediaId START WITH 1 INCREMENT BY 1

SELECT NEXT VALUE FOR dbo.MediaId

SELECT * FROM sys.sequences

ALTER TABLE dbo.Buch
ADD MediaId INT NULL DEFAULT (NEXT VALUE FOR dbo.MediaId)

ALTER TABLE dbo.CD 
ADD MediaId INT NULL DEFAULT (NEXT VALUE FOR dbo.MediaId)

ALTER TABLE dbo.DVD
ADD MediaId INT NULL DEFAULT (NEXT VALUE FOR dbo.MediaId)

INSERT INTO dbo.CD (Titel, Interpret)
VALUES ('Greatest Hits 2', 'Queen')

INSERT INTO dbo.Buch (Titel, Autor)
VALUES ('Datenbanken entwicklen mit SQL Server 2008 Express', 'Robert Panther')

SELECT * FROM dbo.CD
SELECT * FROM dbo.Buch

UPDATE dbo.Buch SET MediaId = NEXT VALUE FOR dbo.MediaId WHERE MediaId IS NULL

SELECT * FROM dbo.Buch

UPDATE dbo.CD SET MediaId = NEXT VALUE FOR dbo.MediaId WHERE MediaId IS NULL
UPDATE dbo.DVD SET MediaId = NEXT VALUE FOR dbo.MediaId WHERE MediaId IS NULL

SELECT * FROM dbo.CD
SELECT * FROM dbo.DVD
