use MediaBase

-- Kapitel 9 --

-- Systemprozeduren

EXECUTE sys.sp_databases

EXEC sys.sp_who
EXEC sys.sp_who2

EXEC sys.sp_renamedb MediaBase, MediaBaseReloaded

-- Dnyamisches SQL

EXEC sys.sp_sqlexec SQLKommando

EXEC sys.sp_sqlexec 'SELECT * FROM dbo.Buch'

DECLARE @SQLCommand AS VARCHAR(255)
DECLARE @SQLTable AS VARCHAR(64)
SET @SQLTable = 'dbo.Buch'
SET @SQLCommand = 'SELECT * FROM ' + @SQLTable
EXEC sys.sp_sqlexec @SQLCommand

-- Systemfunktionen

-- Systemfunktionen: Datumsfunktionen

SELECT Getdate()

SELECT GETDATE() AS Datum, YEAR(GETDATE()) AS Jahr, MONTH(GETDATE()) AS Monat, DAY(GETDATE()) AS Tag

SELECT DATEPART(WEEK, GETDATE()) AS KW

SELECT DATEADD(DAY, 14, GETDATE())

SELECT DATEDIFF(DAY, GETDATE(), '24.12.2012')

SELECT DATEFROMPARTS(2012,12,24)

SELECT DATEDIFF(DAY, GETDATE(), DATEFROMPARTS(YEAR(GETDATE()),12,24))

-- Systemfunktionen: Zeichenkettenfunktionen

SELECT SUBSTRING('ABCDE', 2, 3), LEFT('ABCDE', 2), RIGHT('ABCDE',3)

SELECT Titel, LEN(Titel) FROM dbo.Buch
SELECT Titel, LEN(RTRIM(Titel)) FROM dbo.Buch

SELECT Autor, UPPER(Autor), LOWER(Autor) FROM dbo.Buch

SELECT Interpret, Titel, Titel + '(' + Interpret + ')' AS TitelMitInterpret
FROM dbo.CD

SELECT ISNULL(Zeichenkette, '')

SELECT Interpret, Titel, ISNULL(Titel,'') + '(' + ISNULL(Interpret,'') + ')' AS TitelMitInterpret
FROM dbo.CD

SELECT Interpret, Titel, CONCAT(Titel, '(', Interpret, ')') AS TitelMitInterpret
FROM dbo.CD

-- Systemfunktionen: allgemeine Funktionen

SELECT USER_NAME() AS Benutzer, HOST_NAME() AS Host, APP_NAME() AS Anwendung

SELECT NEWID()

-- Gespeicherte Prozeduren

CREATE PROCEDURE dbo.BuecherAnzeigen AS
BEGIN
	SELECT * FROM dbo.Buch
END
GO

EXEC dbo.BuecherAnzeigen

-- SP mit optionalen Parametern

CREATE PROCEDURE dbo.BuecherVonAutor
	@Autor varchar(80)
AS
BEGIN
	SELECT * FROM dbo.Buch WHERE Autor = @Autor
END
GO

EXEC dbo.BuecherVonAutor 'Robert Panther'

USE [MediaBase]
GO
/****** Object:  StoredProcedure [dbo].[BuecherAnzeigen]    Script Date: 06.04.2012 15:54:21 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER PROCEDURE [dbo].[BuecherAnzeigen] 
	@TitelAnfang varchar(80) = '',
	@AutorAnfang varchar(80) = ''
AS
BEGIN
	SELECT * 
	FROM dbo.Buch 
	WHERE Titel LIKE @TitelAnfang + '%'
		AND Autor LIKE @AutorAnfang + '%'
END
GO

EXEC dbo.BuecherAnzeigen
EXEC dbo.BuecherAnzeigen 'Das'
EXEC dbo.BuecherAnzeigen 'Das', 'Ro'

EXEC dbo.BuecherAnzeigen @AutorAnfang='Ro'
EXEC dbo.BuecherAnzeigen @AutorAnfang='Ro', @TitelAnfang='Das'

-- SP mit OUTPUT-Parameter

USE [MediaBase]
GO
/****** Object:  StoredProcedure [dbo].[BuecherVonAutor]    Script Date: 06.04.2012 15:58:18 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER PROCEDURE [dbo].[BuecherVonAutor]
	@Autor varchar(80),
	@Anzahl int OUTPUT
AS
BEGIN
	SELECT * FROM dbo.Buch WHERE Autor = @Autor
	SET @Anzahl = (SELECT COUNT(*) FROM dbo.Buch WHERE Autor = @Autor)
END
GO

DECLARE @Zeilen AS int
EXEC dbo.BuecherVonAutor 'Robert Panther', @Zeilen OUTPUT
SELECT @Zeilen AS ZeilenAnzahl

-- Skalarwertfunktionen

SELECT Titel, getdate() FROM dbo.Buch

SELECT Titel, Erscheinungsjahr, YEAR(GETDATE())-Erscheinungsjahr AS BuchAlter FROM dbo.Buch

SELECT Titel, Erscheinungsjahr, dbo.UFN_JahreSeit(Erscheinungsjahr) AS BuchAlter FROM dbo.Buch

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Robert Panther
-- Create date: 06.04.2012
-- Description:	ermittelt die Differenz des 
--              aktuellen Datums zu einer 
--              bergebenen Jahreszahl
-- =============================================
CREATE FUNCTION dbo.UFN_JahreSeit
(
	-- Add the parameters for the function here
	@Jahreszahl int
)
RETURNS int
AS
BEGIN
	-- Return the result of the function
	RETURN YEAR(GETDATE())-@Jahreszahl
END
GO

SELECT Titel, Erscheinungsjahr, dbo.UFN_JahreSeit(Erscheinungsjahr) AS BuchAlter 
FROM dbo.Buch

-- Tabellenwertfunktion

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Robert Panther
-- Create date: 06.04.2012
-- Description:	ermittelt alle Tracks zu einer CD
-- =============================================
CREATE FUNCTION dbo.UFN_TracksFromCD
(	
	-- Add the parameters for the function here
	@idCD int
)
RETURNS TABLE 
AS
RETURN 
(
	-- Add the SELECT statement with parameter references here
	SELECT * FROM dbo.CD_Track WHERE idCD=@idCD
)
GO

SELECT * 
FROM dbo.UFN_TracksFromCD(1)

--

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Robert Panther
-- Create date: 06.04.2012
-- Description:	ermittelt alle Tracks zu einer CD
-- =============================================
CREATE FUNCTION dbo.UFN_TracksFromCD2
(	
	-- Add the parameters for the function here
	@idCD int
)
RETURNS @Ergebnis TABLE 
	(ID int, idCD int, CDNr tinyint, Titel varchar(80),
	Interpret varchar(80), Dauer time(7), Bewertung tinyint)
AS
BEGIN
	INSERT INTO @Ergebnis (ID, idCD, CDNr, Titel, Interpret, Dauer, Bewertung)
	SELECT ID, idCD, CDNr, Titel, Interpret, Dauer, Bewertung 
	FROM dbo.CD_Track 
	WHERE idCD=@idCD
	
	RETURN
END
GO

SELECT * FROM dbo.UFN_TracksFromCD2(1)

SELECT * FROM dbo.CD CROSS APPLY dbo.UFN_TracksFromCD(CD.ID)
SELECT * FROM dbo.CD INNER JOIN dbo.CD_Track ON CD.ID=CD_Track.idCD

SELECT * FROM dbo.CD OUTER APPLY dbo.UFN_TracksFromCD(CD.ID)
SELECT * FROM dbo.CD LEFT JOIN dbo.CD_Track ON CD.ID=CD_Track.idCD

-- TRIGGER

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Robert Panther
-- Create date:	06.04.2012
-- Description:	nderungshistorie fr dbo.Buch schreiben
-- =============================================
CREATE TRIGGER dbo.TRG_ChangeLog
   ON  dbo.Buch
   AFTER UPDATE
AS 
BEGIN
	SET NOCOUNT ON;

	INSERT INTO AenderungsLog (Aenderung) VALUES ('Zeile in dbo.Buch gendert!')
END
GO

UPDATE dbo.Buch 
SET Bewertung=3 
WHERE Bewertung IS NULL

--

USE [MediaBase]
GO
/****** Object:  Trigger [dbo]. [TRG_ChangeLog]    Script Date: 06.04.2012 16:24:16 ******/ 
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TRIGGER [dbo].[TRG_ChangeLog]
   ON  [dbo].[Buch]
   AFTER UPDATE, INSERT, DELETE
AS 
BEGIN
	SET NOCOUNT ON;

	INSERT INTO AenderungsLog (Aenderung) 
	VALUES ('Datennderung in Tabelle dbo.Buch!')
END

INSERT INTO dbo.Buch (Titel) VALUES ('TriggerTest')
DELETE FROM dbo.Buch WHERE Titel='TriggerTest'

-- Abfrage der gelschten und hinzugefgten Zeilen

USE [MediaBase]
GO
/****** Object:  Trigger [dbo].[TRG_ChangeLog]    Script Date: 06.04.2012 16:26:35 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TRIGGER [dbo].[TRG_ChangeLog]
   ON  [dbo].[Buch]
   AFTER UPDATE, INSERT, DELETE
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @Aenderungen AS varchar(max)
	
	SET @Aenderungen='Datennderung in Tabelle dbo.Buch: '
		+ (SELECT ltrim(str(count(*))) FROM DELETED) + ' Zeilen gelscht, ' 
		+ (SELECT ltrim(str(count(*))) FROM INSERTED) + ' Zeilen hinzugefgt'
		
	INSERT INTO AenderungsLog (Aenderung) 
	VALUES (@Aenderungen)
END

INSERT INTO dbo.Buch (Titel) VALUES ('TriggerTest')
UPDATE dbo.Buch SET Bewertung=1 WHERE Titel='TriggerTest'
DELETE FROM dbo.Buch WHERE Titel='TriggerTest'

SELECT * FROM dbo.AenderungsLog

-- INSTEAD OF-Trigger

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Robert Panther
-- Create date: 06.04.2012
-- Description:	Lschen von DVDs verhindern
-- =============================================
CREATE TRIGGER TRG_DVDLoeschenVerhindern
   ON dbo.DVD
   INSTEAD OF DELETE
AS 
BEGIN
	SET NOCOUNT ON;
	PRINT 'Lschen abgelehnt!'
END
GO

DELETE FROM dbo.DVD
-- TRUNCATE TABLE dbo.DVD

-- SQL-CURSOR

DECLARE CUR_Buch CURSOR
FOR SELECT ID, ISBN10, ISBN13, Autor, Titel FROM dbo.Buch

DECLARE @ID int, @ISBN10 char(10), @ISBN13 char(14), @Autor varchar(80), @Titel varchar(80)

OPEN CUR_Buch

FETCH NEXT FROM CUR_Buch INTO @ID, @ISBN10, @ISBN13, @Autor, @Titel

WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @Titel

	FETCH NEXT FROM CUR_Buch INTO @ID, @ISBN10, @ISBN13, @Autor, @Titel
END 
CLOSE CUR_Buch
DEALLOCATE CUR_Buch

-- CURSOR und TRIGGER kombiniert verwenden

USE [MediaBase]
GO
/****** Object:  Trigger [dbo].[TRG_ChangeLog]    Script Date: 06.04.2012 16:46:34 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TRIGGER [dbo].[TRG_ChangeLog]
   ON  [dbo].[Buch]
   AFTER UPDATE, INSERT, DELETE
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @Aenderungen AS varchar(max)

	DECLARE @IDsInserted AS varchar(max) = ''
	DECLARE CUR_Inserted CURSOR	FOR SELECT ID FROM INSERTED
	DECLARE @ID int

	OPEN CUR_Inserted
	FETCH NEXT FROM CUR_Inserted INTO @ID

	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @IDsInserted = @IDsInserted + ltrim(str(@ID)) + ', '

		FETCH NEXT FROM CUR_Inserted INTO @ID
	END 
	CLOSE CUR_Inserted
	DEALLOCATE CUR_Inserted
	
	SET @Aenderungen='Hinzugefgte IDs: ' + @IDsInserted

	INSERT INTO AenderungsLog (Aenderung)
	VALUES (@Aenderungen)
END

INSERT INTO dbo.Buch (Titel) VALUES ('TriggerTest1'),('TriggerTest2')
DELETE FROM dbo.Buch WHERE Titel LIKE 'TriggerTest%'
SELECT * FROM dbo.AenderungsLog
