﻿CREATE TABLE [dbo].[Buch] (
    [ID]               INT           IDENTITY (1, 1) NOT NULL,
    [ISBN10]           CHAR (10)     NULL,
    [ISBN13]           CHAR (14)     NULL,
    [Autor]            VARCHAR (80)  NULL,
    [Titel]            VARCHAR (80)  NULL,
    [Verlag]           VARCHAR (80)  NULL,
    [Seiten]           INT           NULL,
    [Auflage]          TINYINT       NULL,
    [Sprache]          VARCHAR (20)  CONSTRAINT [DF_Buch_Sprache] DEFAULT ('deutsch') NULL,
    [Hardcover]        BIT           NULL,
    [Erscheinungsjahr] INT           NULL,
    [Bewertung]        TINYINT       NULL,
    [Beschreibung]     VARCHAR (MAX) NULL,
    [Kategorie]        VARCHAR (80)  NULL,
    [MediaId]          INT           CONSTRAINT [DF_Buch_MediaId] DEFAULT (NEXT VALUE FOR [dbo].[MediaId]) NULL,
    CONSTRAINT [PK_Buch] PRIMARY KEY CLUSTERED ([ID] ASC)
);


GO
CREATE NONCLUSTERED INDEX [IX_Titel]
    ON [dbo].[Buch]([Titel] ASC);


GO
CREATE NONCLUSTERED INDEX [IX_ISBN10]
    ON [dbo].[Buch]([ISBN10] ASC);


GO
CREATE NONCLUSTERED INDEX [IX_ISBN13]
    ON [dbo].[Buch]([ISBN13] ASC);


GO
CREATE NONCLUSTERED INDEX [IX_Autor]
    ON [dbo].[Buch]([Autor] ASC);


GO
CREATE TRIGGER dbo.TRG_ChangeLog
   ON  dbo.Buch
   AFTER UPDATE, INSERT, DELETE
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @Aenderungen AS varchar(max)
	DECLARE @ID int

	DECLARE @IDsInserted AS varchar(max) = ''
	DECLARE CUR_Inserted CURSOR FOR SELECT ID FROM INSERTED

	OPEN CUR_Inserted
	FETCH NEXT FROM CUR_Inserted INTO @ID

	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @IDsInserted = @IDsInserted + ltrim(str(@ID)) + ', '

		FETCH NEXT FROM CUR_Inserted INTO @ID
	END 
	CLOSE CUR_Inserted
	DEALLOCATE CUR_Inserted

	DECLARE @IDsDeleted AS varchar(max) = ''
	DECLARE CUR_Deleted CURSOR FOR SELECT ID FROM DELETED

	OPEN CUR_Deleted
	FETCH NEXT FROM CUR_Deleted INTO @ID

	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @IDsDeleted = @IDsDeleted + ltrim(str(@ID)) + ', '

		FETCH NEXT FROM CUR_Deleted INTO @ID
	END 
	CLOSE CUR_Deleted
	DEALLOCATE CUR_Deleted
	
	SET @Aenderungen='Hinzugefügte IDs: ' + @IDsInserted
		+ ' Gelöschte IDs: ' + @IDsDeleted

	INSERT INTO AenderungsLog (Aenderung)
	VALUES (@Aenderungen)
END
